require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xadcscertificationauthority',
  dscmeta_resource_friendly_name: 'xAdcsCertificationAuthority',
  dscmeta_resource_name: 'MSFT_xAdcsCertificationAuthority',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'xAdcsDeployment',
  dscmeta_module_version: '1.0.0.0',
  docs: 'The DSC xAdcsCertificationAuthority resource type.
         Automatically generated from version 1.0.0.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_logdirectory: {
      type: 'Optional[String]',
      desc: 'Specifies the folder location of the certification authority database log.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_keylength: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Specifies the name of an existing private key container.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_parentca: {
      type: 'Optional[String]',
      desc: 'Specifies the configuration string of the parent certification authority that will certify this CA.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_cacommonname: {
      type: 'Optional[String]',
      desc: 'Specifies the certification authority common name.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ignoreunicode: {
      type: 'Optional[Boolean]',
      desc: 'Specifies that Unicode characters are allowed in certification authority name string.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_catype: {
      type: "Enum['EnterpriseRootCA', 'EnterpriseSubordinateCA', 'StandaloneRootCA', 'StandaloneSubordinateCA']",
      desc: 'Specifies the type of certification authority to install. The possible values are EnterpriseRootCA, EnterpriseSubordinateCA, StandaloneRootCA, or StandaloneSubordinateCA.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_credential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'To install an enterprise certification authority, the computer must be joined to an Active Directory Domain Services domain and a user account that is a member of the Enterprise Admin group is required. To install a standalone certification authority, the computer can be in a workgroup or AD DS domain. If the computer is in a workgroup, a user account that is a member of Administrators is required. If the computer is in an AD DS domain, a user account that is a member of Domain Admins is required.',
      behaviour: :parameter,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_cadistinguishednamesuffix: {
      type: 'Optional[String]',
      desc: 'Specifies the certification authority distinguished name suffix.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: 'Specifies whether the Certificate Authority should be installed or uninstalled.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_validityperiodunits: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Validity period of the certification authority certificate. If this is a subordinate CA, do not specify this parameter because the validity period is determined by the parent CA.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_overwriteexistingdatabase: {
      type: 'Optional[Boolean]',
      desc: 'Specifies that the existing certification authority database should be overwritten.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_certificateid: {
      type: 'Optional[String]',
      desc: 'Specifies the thumbprint or serial number of certification authority certificate.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_outputcertrequestfile: {
      type: 'Optional[String]',
      desc: 'Specifies the folder location for certificate request file.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_keycontainername: {
      type: 'Optional[String]',
      desc: 'Specifies the name of an existing private key container.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_validityperiod: {
      type: "Optional[Enum['Hours', 'Days', 'Months', 'Years']]",
      desc: 'Specifies the validity period of the certification authority certificate in hours, days, weeks, months or years. If this is a subordinate CA, do not use this parameter, because the validity period is determined by the parent CA.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_databasedirectory: {
      type: 'Optional[String]',
      desc: 'Specifies the folder location of the certification authority database.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_certfile: {
      type: 'Optional[String]',
      desc: 'Specifies the file name of certification authority PKCS 12 formatted certificate file.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_overwriteexistingkey: {
      type: 'Optional[Boolean]',
      desc: 'Overwrite existing key container with the same name',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_certfilepassword: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Specifies the password for certification authority certificate file.',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_hashalgorithmname: {
      type: 'Optional[String]',
      desc: 'Specifies the signature hash algorithm used by the certification authority.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_overwriteexistingcainds: {
      type: 'Optional[Boolean]',
      desc: 'Specifies that the computer object in the Active Directory Domain Service domain should be overwritten with the same computer name.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_cryptoprovidername: {
      type: 'Optional[String]',
      desc: 'The name of the cryptographic service provider  or key storage provider that is used to generate or store the private key for the CA.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
